import {
  scaleLinear,
  scaleTime,
  max,
  timeFormat,
  extent,
  histogram as bin,
  timeMonths,
  sum
} from 'd3';

import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const margin = { top: 0, right: 30, bottom: 12, left: 60 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 30;

export const DateHistogram = ({ data, width, height }) => {
  const xValue = d => d['Reported Date'];
  const xAxisLabel = 'Time';

  const yValue = d => d['Total Dead and Missing'];
  const yAxisLabel = 'Total Dead and Missing';

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
	const tickOffset = 5;

  const xAxisTickFormat = timeFormat('%m/%d/%Y');

  
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const [start, stop] = xScale.domain();

  const binnedData = bin()
    .value(xValue)
    .domain(xScale.domain())
    .thresholds(timeMonths(start, stop))(data)
    .map(array => ({
      y: sum(array, yValue),
      x0: array.x0,
      x1: array.x1
    }));
  
  const yScale = scaleLinear()
    .domain([0, max(binnedData, d => d.y)])
    .range([innerHeight, 0]);
  
  console.log("X Scale Ticks:", xScale.ticks(5));
    console.log("Y Scale Ticks:", yScale.domain());
  
  return (
    <>
  	<rect width={width} height={height} fill='white'/>
    <g transform={`translate(${margin.left},${margin.top})`}>
      <AxisBottom 
          xScale={xScale} 
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={tickOffset}
          />
        <text 
          className="axis-label" 
          x={innerWidth / 2} 
          y={innerHeight + xAxisLabelOffset} 
          textAnchor="middle">
          {xAxisLabel}
        </text>
        <AxisLeft 
          yScale={yScale} 
          innerWidth={innerWidth} 
          />
        <text 
          className="axis-label" 
          textAnchor="middle"
          transform={` translate(${-yAxisLabelOffset},${innerHeight / 2}),rotate(-90)`}
          >
          {yAxisLabel}
        </text>
      <Marks
        binnedData={binnedData}
        xScale={xScale}
        yScale={yScale}
        tooltipFormat={d => d}
        circleRadius={2}
        innerHeight={innerHeight}
      />
    	</g>
    </>
  );
};